@@ -16,9 +16,9 @@ from typing import Dict, Union, List, Iterable, Optional
 from rdflib import Literal
 import re
 
-from . import auxiliary as aux
-from . import settings
-from .auxiliary import (
+from pyerk import auxiliary as aux
+from pyerk import settings
+from  pyerk.auxiliary import (
     InvalidURIError,
     InvalidPrefixError,
     PyERKError,
@@ -89,16 +89,20 @@ class Entity(abc.ABC):
         self.uri = None  # will be set in __post_init__
 
     def __call__(self, *args, **kwargs):
-        # returning self allows to use I1234 and I1234("human readable item name") interchageably
-        # (once the object is created)
-
+        
         custom_call_method = getattr(self, "_custom_call", None)
         if custom_call_method is None:
             msg = f"entity {self} has not defined a _custom_call-method and thus cannot be called"
             raise TypeError(msg)
         else:
             assert callable(custom_call_method)
-            return custom_call_method(*args, **kwargs)
+            
+            res = custom_call_method(*args, **kwargs)
+            
+            if custom_call_post_process := getattr(self, "_custom_call_post_process", None):
+                res = custom_call_post_process(res, *args, **kwargs)
+            
+            return res
 
     def idoc(self, adhoc_label: str):
         """
@@ -151,8 +155,8 @@ class Entity(abc.ABC):
             pass
         try:
             res = process_key_str(attr_name)
-        except KeyError as err:
-            # this happens if a syntactically valid short key could not be resolved
+        except (KeyError, aux.UnknownURIError) as err:
+            # this happens if a syntactically valid key string could not be resolved
             raise AttributeError(*err.args)
         if not res.etype == EType.RELATION:
             r3 = getattr(self, "R3", None)
@@ -621,7 +625,7 @@ class DataStore:
         :return:            corresponding entity
         """
 
-        processed_key = process_key_str(key_str)
+        processed_key = process_key_str(key_str, mod_uri=mod_uri)
         assert processed_key.etype in (EType.ITEM, EType.RELATION)
 
         if mod_uri is None:
@@ -870,7 +874,9 @@ re_suffix_underscore = re.compile(r"^__([\w\-]+)$")  # \w means alphanumeric (in
 re_suffix_square_brackets = re.compile(r"""^\[["'](.+)["']\]""")
 
 
-def process_key_str(key_str: str, check: bool = True, resolve_prefix: bool = True) -> ProcessedStmtKey:
+def process_key_str(
+        key_str: str, check: bool = True, resolve_prefix: bool = True, mod_uri: str = None,
+) -> ProcessedStmtKey:
     """
     In ERK there are the following kinds of keys:
         - a) short_key like `R1234`
@@ -881,6 +887,8 @@ def process_key_str(key_str: str, check: bool = True, resolve_prefix: bool = Tru
         - e) index-labeld key like  `R1234["my relation"]`
         - f) prefixed index-labeld key like  `bi__R1234["my relation"]`
 
+    See also: userdoc/overview.html#keys-in-pyerk
+    
     Also, the leading character indicates the entity type (EType).
 
     This function expects any of these cases.
@@ -888,6 +896,7 @@ def process_key_str(key_str: str, check: bool = True, resolve_prefix: bool = Tru
     :param check:       boolean flag; determines if the label part should be checked wrt its consistency to
     :param resolve_prefix:
                         boolean flag; determines if
+    :param mod_uri:     optional uri of the module
 
 
     :return:            a data structure which allows to access short_key, type and label separately
@@ -940,7 +949,7 @@ def process_key_str(key_str: str, check: bool = True, resolve_prefix: bool = Tru
         raise KeyError(msg)
 
     if resolve_prefix:
-        _resolve_prefix(res)
+        _resolve_prefix(res, passed_mod_uri=mod_uri)
 
     if check:
         aux.ensure_valid_short_key(res.short_key)
@@ -949,20 +958,33 @@ def process_key_str(key_str: str, check: bool = True, resolve_prefix: bool = Tru
     return res
 
 
-def _resolve_prefix(pr_key: ProcessedStmtKey) -> None:
-    # get uri from prefix
-    mod_uri = get_active_mod_uri(strict=False)
+def _resolve_prefix(pr_key: ProcessedStmtKey, passed_mod_uri: str = None) -> None:
+    """
+    get uri from prefix or from passed argument or from active module
+    """
+    active_mod_uri = get_active_mod_uri(strict=False)
 
     if pr_key.prefix is None:
-        if mod_uri is None:
-            # assume that `builtin_entities` is meant
-            mod_uri = settings.BUILTINS_URI
+        if active_mod_uri is None:
+            if passed_mod_uri:
+                mod_uri = passed_mod_uri
+            else:
+                # assume that `builtin_entities` is meant
+                mod_uri = settings.BUILTINS_URI
         else:
             # Situation: create_item(..., R321="some value") within an active module
             # (no prefix). short_key R321 could refer to active module or to builtin_entities -> search in this order
 
-            # 1. check active mod
-            candidate_uri = aux.make_uri(mod_uri, pr_key.short_key)
+            # 1. check that passed_mod_uri does not contradict
+            if passed_mod_uri and (passed_mod_uri != active_mod_uri):
+                msg = (
+                    f"encountered inconsistent uris for object with key_str {pr_key.original_key_str}. "
+                    f"from active mod: '{active_mod_uri}' vs explicitly passed: '{passed_mod_uri}'."
+                )
+                raise aux.InvalidURIError(msg)
+
+            # 2. check active mod
+            candidate_uri = aux.make_uri(active_mod_uri, pr_key.short_key)
             res_entity = ds.get_entity_by_uri(candidate_uri, strict=False)
 
             if res_entity is not None:
@@ -973,20 +995,27 @@ def _resolve_prefix(pr_key: ProcessedStmtKey) -> None:
             candidate_uri = aux.make_uri(settings.BUILTINS_URI, pr_key.short_key)
             res_entity = ds.get_entity_by_uri(candidate_uri, strict=False)
 
-            # if res_entity is still None no entity could be found
             if res_entity is not None:
                 pr_key.uri = candidate_uri
                 return
             else:
                 # if res_entity is still None no entity could be found
                 msg = (
-                    f"No entity could be found for short_key {pr_key.short_key}, neither in active module ({mod_uri}) "
-                    f"nor in builin_entities ({settings.BUILTINS_URI})"
+                    f"No entity could be found for short_key {pr_key.short_key}, neither in active module "
+                    f"({active_mod_uri}) nor in builin_entities ({settings.BUILTINS_URI})"
                 )
                 raise KeyError(msg)
     else:
+        # prefix was not not None
         mod_uri = ds.get_uri_for_prefix(pr_key.prefix)
 
+        if passed_mod_uri and (passed_mod_uri != active_mod_uri):
+            msg = (
+                f"encountered inconsistent uris for object with key_str {pr_key.original_key_str}. "
+                f"from prefix mod: '{mod_uri}' vs explicitly passed: '{passed_mod_uri}'."
+            )
+            raise aux.InvalidURIError(msg)
+
     pr_key.uri = aux.make_uri(mod_uri, pr_key.short_key)
 
 
@@ -1004,16 +1033,18 @@ def check_processed_key_label(pkey: ProcessedStmtKey) -> None:
         return
 
     try:
-        entity = ds.get_entity_by_key_str(pkey.short_key)
+        entity = ds.get_entity_by_uri(pkey.uri)
     except KeyError:
         # entity does not exist -> no label to compare with
         return
-    label_compare_str = entity.R1.replace(" ", "_")
+    label_compare_str1 = entity.R1
+    label_compare_str2 = entity.R1.replace(" ", "_")
 
-    if label_compare_str.lower() != pkey.label.lower():
+    if pkey.label.lower() not in (label_compare_str1.lower(), label_compare_str2.lower()):
         msg = (
-            f'check of label consistency failed for key {pkey.original_key_str}. Expected:  "{label_compare_str}"  '
-            f'but got   "{pkey.label}"  . Note: this test is *not* case-sensitive.'
+            f'check of label consistency failed for key {pkey.original_key_str}. Expected:  one of '
+            f'("{label_compare_str1}", "{label_compare_str2}") but got  "{pkey.label}". '
+            'Note: this test is *not* case-sensitive.'
         )
         raise ValueError(msg)
 
@@ -1095,7 +1126,11 @@ def create_item(key_str: str = "", **kwargs) -> Item:
             msg = f"unexpected key: {dict_key} during creation of item {item_key}."
             raise ValueError(msg)
 
-        new_kwargs[processed_key.short_key] = value
+        if processed_key.prefix:
+            new_key = f"{processed_key.prefix}__{processed_key.short_key}"
+        else:
+            new_key = processed_key.short_key
+        new_kwargs[new_key] = value
 
     itm = Item(base_uri=mod_uri, key_str=item_key, **new_kwargs)
     assert itm.uri not in ds.items, f"Problematic (duplicated) uri: {itm.uri}"
@@ -1678,8 +1713,9 @@ class uri_context:
     Context manager for creating entities with a given uri
     """
 
-    def __init__(self, uri: str):
+    def __init__(self, uri: str, prefix: str = None):
         self.uri = uri
+        self.prefix = prefix
 
     def __enter__(self):
         """
@@ -1687,6 +1723,9 @@ class uri_context:
         :return:
         """
         _uri_stack.append(self.uri)
+
+        if self.prefix:
+            ds.uri_prefix_mapping.add_pair(self.uri, self.prefix)
         return self
 
     def __exit__(self, exc_type, exc_val, exc_tb):
@@ -1694,6 +1733,8 @@ class uri_context:
 
         res = _uri_stack.pop()
         assert res == self.uri
+        if self.prefix:
+            ds.uri_prefix_mapping.remove_pair(self.uri, self.prefix)
 
 
 def unload_mod(mod_uri: str, strict=True) -> None:
